from OFS import Image as ofsimage
_marker = object()

class FileProperty(object):
    """Stores the given file data in a zodb safe way.
    """

    def __init__(self, field, name=None):
        if name is None:
            name = field.__name__

        self.__field = field
        self.__name = name

    def __get__(self, inst, klass):
        if inst is None:
            return self

        value = inst.__dict__.get(self.__name, _marker)
        if value is _marker:
            field = self.__field.bind(inst)
            value = getattr(field, 'default', _marker)
            if value is _marker:
                raise AttributeError(self.__name)

        return value

    def __set__(self, inst, value):
        field = self.__field.bind(inst)
        if field.readonly and inst.__dict__.has_key(self.__name):
            raise ValueError(self.__name, 'field is readonly')

        if inst.__dict__.get(self.__name, None) is None:
            inst.__dict__[self.__name] = ofsimage.File(self.__name, 
                                                       self.__name, '')
        
        obj = inst.__dict__[self.__name]
        obj.manage_upload(file=value)
        inst._p_changed = True

    def __getattr__(self, name):
        return getattr(self.__field, name)





from xml.sax import saxutils
from zope.app.form import browser
from zope.app.form.browser import widget
from zope.app.form import interfaces as forminterfaces
from zope.cachedescriptors.property import Lazy

class FileDownloadWidget(browser.DisplayWidget):
    """Widget capable of downloading file.
    """
    
    required = False
    
    @property
    def base_url(self):
        contentobj = self.context.context.context
        return contentobj.absolute_url() + '/downloadfile'

    @property
    def filename(self):
        filename = getattr(self, '_filename', None)
        if filename is not None:
            return filename

        if self._renderedValueSet():
            value = self._data
        else:
            # when tracing we come here, and value ends up None, or whatever
            value = self.context.default
        #...and then this condition is a match, and we get None returned.
        #if value == self.context.missing_value:
        #    return None
        self._filename = saxutils.escape(getattr(value, '__name__', ''))

        return self._filename
    
    @property
    def url(self):
        filename = self.filename
        field = self.context
        obj = field.context
        contentobj = obj.context

        ifname = field.interface.__identifier__
        pos = ifname.rfind('.')
        ifpackage = ''
        if pos > -1:
            ifpackage = ifname[:pos]
            ifname = ifname[pos+1:]
        
        url = '%s?field=%s:%s:%s' % (self.base_url,
                                     ifpackage,
                                     ifname,
                                     field.getName())
        return url

    def __call__(self):
        url = self.url
        filename = self.filename

        return widget.renderElement(
            u'a',
            href=url,
            contents=filename)

class FileUploadWidget(widget.SimpleInputWidget):

    @Lazy
    def _modified_name(self):
        return "_modify_%s" % self.name

    def __call__(self):
        kwargs = {'name': self.name,
                  'filename': '',
                  'modified_name': self._modified_name}

        if self._data is not None:
            filename = getattr(self._data,'filename', '')
            if not filename:
                filename = getattr(self._data,'__name__', '')
            kwargs['filename'] = filename
            
            options = """
            <div><input type="radio" name="%(modified_name)s:int" value="0" checked="checked" /> Keep existing file '%(filename)s'</div>
            <div><input type="radio" name="%(modified_name)s:int" value="1" /> Replace with new file:</div>
            """ % kwargs
        elif self.context.required:
            options = """
            <input type="hidden" name="%(modified_name)s:int" value="1" />
            """ % kwargs
        else:
            options = """
            <div><input type="radio" name="%(modified_name)s:int" value="0" checked="checked" /> No file </div>
            <div><input type="radio" name="%(modified_name)s:int" value="1" /> New file:</div>
            """ % kwargs           
        
        kwargs['options'] = options
        
        return """
            %(options)s
            <div class="_file_%(name)s"><input type="file" name=%(name)s /></div>
        """ % kwargs

    def _getFormInput(self):
        modify = int(self.request.get('_modify_%s' % self.name, 0))
        if not modify:
            return None

        v = self.request.get(self.name, None) or None

        return v

    def hasInput(self):
        return bool(self.request.get(self._modified_name, 0))



from zope.app.form.browser import widget
from p4a.fileimage import file

class ImageDisplayWidget(file.FileDownloadWidget):
    """Widget capable of displaying an image file.
    """
    
    @property
    def base_url(self):
        contentobj = self.context.context.context
        return contentobj.absolute_url() + '/viewimage'

    def __call__(self):
        if not self._data:
            return widget.renderElement(u'span',
                                        cssClass='image-absent',
                                        contents='No image set')
        
        url = self.url
        
        field = self.context
        extra = {}

#        XXX We do not want preferred_dimensions. Get the actual dimensions!
#            But, how is that done?         
#        if field.preferred_dimensions is not None:
#            extra['width'], extra['height'] = field.preferred_dimensions

        return widget.renderElement(
            u'img',
            src=url,
            **extra)

class ImageURLWidget(file.FileDownloadWidget):
    """Widget that returns the URL of the image
       This is clearly overkill, but it was the easy way to get
       something working fast.
       Revisit the consumer of this class and probably 
       access the url inline there.
    """
    
    @property
    def base_url(self):
        contentobj = self.context.context.context
        return contentobj.absolute_url() + '/viewimage'

    def __call__(self):
        if not self._data:
            return widget.renderElement(u'span',
                                        cssClass='image-absent',
                                        contents='No image set')
        
        return self.url
        



import os
from p4a.fileimage import utils

class DownloadFile(object):
    """A view for downloading file content.
    """

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        ifpackagename, ifname, fieldname = \
                       self.request.form.get('field', ':').split(':')
        ifpackage = __import__(ifpackagename, {}, {}, ifpackagename)
        iface = getattr(ifpackage, ifname)
        adapted = iface(self.context)
        value = getattr(adapted, fieldname)

        return value.index_html(self.request, self.request.response)

class ViewImage(DownloadFile):
    """A view for streaming image content.
    """



import unittest

from zope.testing import doctest
from zope.component import testing

def test_suite():
    return unittest.TestSuite((
        doctest.DocTestSuite('p4a.fileimage._property'),
                               
        doctest.DocFileSuite('file.txt',
                             setUp=testing.setUp,
                             tearDown=testing.tearDown,
                             ),
        ))



import os
import tempfile

def write_to_tempfile(fileobj):
    """Assumes the file obj is of type OFS.Image.File and will write
    it to a temporary file returning the filename of the temp file.  This
    writing mechanism uses raw attributes knowing the data may be of
    type pdata.
    """
    
    id = fileobj.id
    if callable(id):
        id = id()
    fd, filename = tempfile.mkstemp('_'+id)
    os.close(fd)
    fout = open(filename, 'wb')
    
    data = fileobj.data
    if isinstance(data, str):
        fout.write(data)
    else:
        pdata = data
        while pdata is not None:
            fout.write(pdata.data)
            pdata = pdata.next
    fout.close()

    return filename

def write_ofsfile_to_tempfile(obj, preferred_name=None):
    """Assumes the file obj is of type OFS.Image.File and will write
    it to a temporary file returning the filename of the temp file.  Uses
    the possibly acquired index_html method to fetch the file.  This
    is a little more compatible with objects that seem like OFS.Image.File
    instances.
    """

    filename = preferred_name
    if filename is None:
        filename = obj.id
        if callable(filename):
            filename = filename()
    fd, filename = tempfile.mkstemp('_'+filename)
    os.close(fd)
    fout = open(filename, 'wb')
    
    class TempResponse(object):
        def setHeader(self, n, v):
            pass
        def setBase(self, v):
            pass
        def write(self, d):
            fout.write(d)
    
    class TempRequest(object):
        def get_header(self, n, default=None):
            if default is not None:
                return default
            return ''
    
    res = obj.index_html(TempRequest(), TempResponse())
    if res:
        if isinstance(res, str):
            fout.write(res)
        else:
            # assumes some sort of iterator
            for x in res:
                fout.write(x)

    fout.close()

    return filename



from zope import schema
from zope import component
from p4a.fileimage import file

class ImageField(file.FileField):
    """A field for representing an image.
    """

    def __init__(self, preferred_dimensions=None, **kw):
        super(ImageField, self).__init__(**kw)

        self.preferred_dimensions = preferred_dimensions




_marker = object()

class DictProperty(object):
    """Computed fields based on a named dict on the context object.
    
    To demonstrate, we first build a sample schema.
      
      >>> from zope import interface, schema
      >>> class IFoo(interface.Interface):
      ...     text = schema.TextLine(title=u'Some Text')
      
    Next we construct a class which implements the schema and uses our
    *DictProperty*.
      
      >>> class Foo(object):
      ...     interface.implements(IFoo)
      ...     text = DictProperty(IFoo['text'], 'somedict')
      
    Lets instantiate our test class and try setting/getting the value
    of the *DictProperty* field.
      
      >>> foo = Foo()
      >>> foo.text = u'abc'
      >>> foo.text
      u'abc'
  
    Now lets make sure the internal dict was properly updated.
      
      >>> foo.somedict['text']
      u'abc'
      
    The reverse should also work, try updating the internal dict and
    make sure the field was updated properly.
    
      >>> foo.somedict['text'] = u'updated foo'
      >>> foo.text
      u'updated foo'
      
    """

    def __init__(self, field, attrname, name=None):
        if name is None:
            name = field.__name__

        self.__field = field
        self.__attrname = attrname
        self.__name = name
    
    def __get__(self, inst, _class):
        if inst is None:
            return self        

        d = inst.__dict__.get(self.__attrname, None)
        if d is None:
            return None
        
        value = d.get(self.__name, _marker)
        if value is _marker:
            field = self.__field.bind(inst)
            value = getattr(field, 'default', _marker)
            if value is _marker:
                raise AttributeError(self.__name)

        return value        

    def __set__(self, inst, value):
        field = self.__field.bind(inst)
        field.validate(value)

        d = inst.__dict__.get(self.__attrname, None)
        if d is None:
            d = {}
        
        if field.readonly and d.has_key(self.__name):
            raise ValueError(self.__name, 'field is readonly')
        
        d[self.__name] = value
        inst.__dict__[self.__attrname] = d

    def __getattr__(self, name):
        return getattr(self.__field, name)



from zope import schema
from zope import component

class FileField(schema.Field):

    def __init__(self, **kw):
        super(FileField, self).__init__(**kw)

    def _validate(self, value):
        super(FileField, self)._validate(value)



from setuptools import setup, find_packages
import sys, os

version = '1.0'
readme = open('README.txt')
long_description = readme.read()
readme.close()

setup(name='p4a.fileimage',
      version=version,
      description="File/Image widget for Zope 3",
      long_description=long_description,
      classifiers=[
          'Framework :: Zope3',
          'Programming Language :: Python',
          'License :: OSI Approved :: GNU General Public License (GPL)',
          'Topic :: Software Development :: Libraries :: Python Modules',
          ],
      keywords='',
      author='Rocky Burt',
      author_email='rocky@serverzen.com',
      url="",
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['p4a'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )
